import { OnInit, Input, Output, ChangeDetectorRef, EventEmitter } from '@angular/core';
import { FormControl } from '@angular/forms';
import { FriendlyValidationErrorsService } from './../../services/friendly-validation-errors.service';
// import { Subject } from 'rxjs';

export abstract class NativeInputComponent implements OnInit {

	@Input()
	control: FormControl;

	@Input()
	set meta(meta: StringMap<any>) {
		this._meta = meta;
		this.exposeForTemplate();
	}

	@Output()
	call: EventEmitter<string> = new EventEmitter<string>();

	readonly componentName: string;
	exposeMetaInTemplate: string[] = [];
	_meta: StringMap<any>;

	pendingValidation: boolean;

	// firstFailureMsg: Subject<string> = new Subject();

	constructor(
		protected valErrsService: FriendlyValidationErrorsService,
		protected _cdr: ChangeDetectorRef
	) {}

	ngOnInit() {
		// this.control is not an instance of FormControl when it's a Checkbutton in a Checkbutton group (but is when it's a solo Checkbutton) - BUT WHY? WHY!!!
		// Hence the 'if' statement
		if (this.control instanceof FormControl) {
			this.control.valueChanges.subscribe(this.markForCheck.bind(this));
			this.control.statusChanges.subscribe(status => {
				if (status === 'PENDING') {
					this.pendingValidation = true;
				} else if (this.pendingValidation) {
					this.pendingValidation = false;
					this.markForCheck();
				}
			});
		}
	}

	exposeForTemplate() {
		// Move meta variables up a level, for direct access in templates
		this.exposeMetaInTemplate.map(p => this[p] = this._meta[p] !== undefined ? this._meta[p] : this[p]);
	}

	markForCheck() {
		if (this._cdr) {
			this._cdr.markForCheck();
		}
	}

	handle(fnId: string, val: any): void {
		this.call.emit(fnId); // Add control's current value
	}

	handleChange(): void {
		if (this._meta.change) {
			this.handle(this._meta.change, this.control.value);
		}
	}

	getName(): string {
		return this.componentName;
	}

	getFirstFailureMsg(): string {
		if (!this.control.errors || this.control.errors === {}) {
			return '';
		}
		const key = Object.keys(this.control.errors)[0];
		return this._meta.valFailureMsgs[key] || this.valErrsService.getFriendly(key, this.control.errors[key]);
	}

	/*
	getFirstFailureMsg(): void {
		if (!this.control.errors || this.control.errors === {}) {
			this.firstFailureMsg.next('');
		}
		console.log(this.control.errors);
		const key = Object.keys(this.control.errors)[0];
		const err = this._meta.valFailureMsgs[key] || this.valErrsService.getFriendly(key, this.control.errors[key]);
		this.firstFailureMsg.next(err);
	}
	*/


}
